#include <stdio.h>
#include <stdlib.h>

#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/sensors/SoTimerSensor.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoQuadMesh.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoFaceSet.h>

/* Verticies of the composite polygons of the obelisk */
static float vertices[32][3] = {				// 32 3-dimensional coordinates

	/* triangular pyramid at top of obelisk */
	{ 0, 30, 0}, {-2,27, 2}, { 2,27, 2},			// Front Triangle
	{ 0, 30, 0}, {-2,27,-2}, {-2,27, 2},			// Left Triangle
	{ 0, 30, 0}, { 2,27,-2}, {-2,27,-2},			// Rear Triangle
	{ 0, 30, 0}, { 2,27, 2}, { 2,27,-2},			// Right Triangle

	/* quadrilateral sides of the obelisk */
	{-2, 27, 2}, {-4,0, 4}, { 4,0, 4}, { 2,27, 2},		// Front Quadrilateral
	{-2, 27,-2}, {-4,0,-4}, {-4,0, 4}, {-2,27, 2},		// Left  Quadrilateral
	{ 2, 27,-2}, { 4,0,-4}, {-4,0,-4}, {-2,27,-2},		// Rear  Quadrilateral
	{ 2, 27, 2}, { 4,0, 4}, { 4,0,-4}, { 2,27,-2},		// Right Quadrilateral

	/* square bottom of obelisk */
	{ 4, 0, 4}, { 4,0, -4}, { -4,0,-4}, { -4,0,4}		// Square bottom
};

/* Number of vertices in each polygon */
static int numvertices[9] = {/* pyramid */ 3, 3, 3, 3, /* faces */ 4, 4, 4, 4, /* bottom */ 4};

static float polygonNormals[9][3] = {	// Normals from the center of each polygon
	{0, .555,  .832},	// Front Triangle
	{-.832, .555, 0},	// Left Triangle
	{0, .555, -.832},	// Rear Triangle
	{ .832, .555, 0},	// Right Triangle

	{0, .0739,  .9973},	// Front Quadrilateral
	{-.9972, .0739, 0},	// Left Quadrilateral
	{0, .0739, -.9973},	// Rear Quadrilateral
	{ .9972, .0739, 0},	// Right Quadrilateral

	{0, -1, 0},		// Base
};

SoSeparator *makeObelisk() { // Obelisk Generation Function
	SoSeparator *obelisk;
	SoCoordinate3 *coordinates;

	SoFaceSet *faces;
	SoMaterial *color;
	SoNormal *normals;
	SoNormalBinding *normalBinding;

	obelisk = new SoSeparator();
	obelisk->ref();

	/* Set the color of the Obelisk */
	color = new SoMaterial;
	color->diffuseColor.setValue(.4, .4, .4);
	obelisk->addChild(color);

	/* Map the verticies array to 3-dimensional coordinates */
	coordinates = new SoCoordinate3;
	coordinates->point.setValues(0, 32, vertices);
	obelisk->addChild(coordinates);

	/* Map the polygonNormals array to the polygon normals */
	normals = new SoNormal;
	normals->vector.setValues(0, 9, polygonNormals);
	obelisk->addChild(normals);
	normalBinding = new SoNormalBinding;
	normalBinding->value = SoNormalBinding::PER_FACE;
	obelisk->addChild(normalBinding);

	/* Create the Obelisk using the above coordinates */
	faces = new SoFaceSet;
	faces->numVertices.setValues(0, 9, numvertices);
	obelisk->addChild(faces);

	obelisk->unrefNoDelete();
	return obelisk;
}

int main(int argc, char **argv) {
	Widget window;
	SoSeparator *root;
	SoSeparator *obelisk;
	SoXtExaminerViewer *viewer;

	/* Initialize Inventor and Xt; create new viewer */
	window = SoXt::init(*argv);
	viewer = new SoXtExaminerViewer(window);

	/* Generate the Obelisk */
	root = new SoSeparator;
	if (!(obelisk = makeObelisk())) {
		printf("Error: Unable to generate to obelisk\n");
		exit(1);
	}
	root->addChild(obelisk);

	/* Display SceneGraph in viewer */
	viewer->setSceneGraph(root);
	viewer->show();

	/* Display viewer window and initialize event handler */
	SoXt::show(window);
	SoXt::mainLoop();
}
